import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.IOException;

import javax.media.ControllerEvent;
import javax.media.ControllerListener;
import javax.media.EndOfMediaEvent;
import javax.media.Manager;
import javax.media.MediaLocator;
import javax.media.NoPlayerException;
import javax.media.Player;
import javax.media.RealizeCompleteEvent;
import javax.media.Time;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JPanel;

/**
 * JFrame containing a simple player that listens to 
 * ControllerEvents (processing chain related events), as well as 
 * ActionEvents (initiated by the user through button clicks etc.).
 */
public class SimplePlayerOwnControls extends JFrame implements ControllerListener, ActionListener {
	
	/** serialVersionUID is needed because this class is serializable.*/
	private static final long serialVersionUID = 1L;
	/** Player (processing chain restricted to rendering streams)*/
	Player player = null;
	/** Play button */
	JButton btnPlay;
	/** Stop button */
	JButton btnStop;
	/** Forward button */
	JButton btnForward;
	/** Rewind button */
	JButton btnBackward;
	/** Boolean */
	boolean isPlaying;
	
	/** 
	 * Creates a new SimplePlayer including controls and 
	 * automatically loads the specified video file
	 * @param filename name of the video file 	 
	 */
	public SimplePlayerOwnControls(String filename){
		
		// basic window functionality
		setTitle("Simple Player");
		addWindowListener(new WindowAdapter() {
			@Override
			public void windowClosing(WindowEvent event) {
				player.stop();
				player.deallocate();
				System.exit(0);
			}
		});
		setSize(400,300);
		setLayout(new BorderLayout());
		
		// create player and control listener
		// catch exceptions in case something goes wrong
		try {
			player = Manager.createPlayer(new MediaLocator(filename));
			player.addControllerListener(this);
			player.realize();
		} catch (NoPlayerException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		// create buttons 
		btnPlay = new JButton("||");
		btnPlay.addActionListener(this);
		btnStop = new JButton("Stop");
		btnStop.addActionListener(this);
		btnForward = new JButton(">>");
		btnForward.addActionListener(this);
		btnBackward = new JButton("<<");
		btnBackward.addActionListener(this);
		
		// add the buttons to the button panel
		JPanel buttonPanel = new JPanel();
		buttonPanel.add(btnPlay);
		buttonPanel.add(btnStop);
		buttonPanel.add(btnBackward);
		buttonPanel.add(btnForward);
		add(buttonPanel, BorderLayout.SOUTH);
	}
	
	/**
	 * Incoming events (fired by the processing chain) are sent to this method
	 * @param event incoming event
	 */
	public void controllerUpdate(ControllerEvent e) {
		// handle incoming RealizeCompleteEvents
		if (e instanceof RealizeCompleteEvent){
			
			add(player.getVisualComponent(), BorderLayout.CENTER);
			//add(p.getControlPanelComponent(), BorderLayout.SOUTH);
			player.start();
			isPlaying = true;
			setVisible(true);	
		}

		// handle incoming EndOfMediaEvents
		else if (e instanceof EndOfMediaEvent) {

			player.stop();
			player.setMediaTime(new Time(0));
			player.start();
		}
	}

	/**
	 * Incoming action events (button clicks etc.) are sent to this method
	 * @param e action event
	 */
	public void actionPerformed(ActionEvent e) {
		Object source = e.getSource();
		
		// if play button is clicked
		if (source == btnPlay){
			// if the video was playing already
			if (isPlaying){
				player.stop(); // stop the video
				isPlaying = false;
				btnPlay.setText("Play"); // change label
			}
			// if the video was not playing
			else {
				player.start(); // start the video
				isPlaying = true;
				btnPlay.setText("||"); // change label
			}
			
		}
		// if stop button is clicked
		else if (source == btnStop){
			player.stop(); // stop the video
			player.setMediaTime(new Time(0)); // reset time
			btnPlay.setText("Play"); // change label
			
		}
		// if back button is clicked
		else if (source == btnBackward){
			player.stop(); // stop the video
			
			// go back one second
			double newTime = player.getMediaTime().getSeconds()-(1); 
			if (newTime >= 0){
				player.setMediaTime(new Time(newTime)); 
			}
			
		}
		// if forward button is clicked
		else if (source == btnForward){
			player.stop();
			
				// go one second ahead
				double newTime = player.getMediaTime().getSeconds()+(1);
				if (newTime < player.getDuration().getSeconds()){
					player.setMediaTime(new Time(newTime));
				}
				else{
					player.setMediaTime(new Time(0));
				}
		}
	}
	
	public static void main(String[] args){
		new SimplePlayerOwnControls("http://www.medien.ifi.lmu.de/lehre/ss08/mt/uebungen/blatt4video.avi");
	}
}
