import java.awt.Dimension;

import javax.media.Buffer;
import javax.media.Effect;
import javax.media.Format;
import javax.media.ResourceUnavailableException;
import javax.media.format.RGBFormat;

/**
 * @author Horst Eidenberger (hme@ims.tuwien.ac.at), adaptation: Heinrich
 *         Hussmann, Bettina Conradi
 * @version 1
 */

public class BlendEffect implements Effect {

	private Format inputFormat;
	private Format outputFormat;
	/** Supported input formats */
	private Format[] inputFormats;
	/** Supported output formats */
	private Format[] outputFormats;
	
	int frameCount = 100; // default
	int count = 1;
	
	public BlendEffect(int frameCount) {
		this.frameCount = frameCount;
		
		// specify supported input formats
		inputFormats = new Format[] { new RGBFormat(null, Format.NOT_SPECIFIED,
				Format.intArray, Format.NOT_SPECIFIED, 32,
				Format.NOT_SPECIFIED, Format.NOT_SPECIFIED,
				Format.NOT_SPECIFIED) };

		// specify supported output formats
		outputFormats = new Format[] { new RGBFormat(null,
				Format.NOT_SPECIFIED, Format.intArray, Format.NOT_SPECIFIED,
				32, Format.NOT_SPECIFIED, Format.NOT_SPECIFIED,
				Format.NOT_SPECIFIED) };
	}

	public BlendEffect() {		
		
		// specify supported input formats
		inputFormats = new Format[] { new RGBFormat(null, Format.NOT_SPECIFIED,
				Format.intArray, Format.NOT_SPECIFIED, 32,
				Format.NOT_SPECIFIED, Format.NOT_SPECIFIED,
				Format.NOT_SPECIFIED) };

		// specify supported output formats
		outputFormats = new Format[] { new RGBFormat(null,
				Format.NOT_SPECIFIED, Format.intArray, Format.NOT_SPECIFIED,
				32, Format.NOT_SPECIFIED, Format.NOT_SPECIFIED,
				Format.NOT_SPECIFIED) };
	}
	
	public synchronized int process(Buffer in, Buffer out) {
		// copy input buffer to output buffer
		out.copy(in);
		int[] data = (int[]) out.getData();

		//get height and width of current frame
		RGBFormat inFormat = (RGBFormat) in.getFormat();
		Dimension inSize = inFormat.getSize();
		int width = (int) inSize.getWidth();
		int height = (int) inSize.getHeight();

		// pixel coordinates
		int x, y;
		// position of the pixel in the data[]
		int offBuffer;

		// run through all pixels
		for(y=0; y<height; y++) {
			 for(x=0; x<width; x++) {

				 // calculate position of the pixel in the data[]
				 offBuffer = y*width + x;
				 
				 // get RGB value of current pixel
			     int p = data[offBuffer];

			     // extract values for red, green and blue from RGB value
			     // by doing a bit shift
			     int red 	= (p >> 16) &0xff;
			     int green	= (p >> 8) &0xff;
			     int blue	= p &0xff;

			     // blend factor (increasing from 1/100 to 1)
				 double factor = ((double)count)/frameCount;
				 
			     // calculate new red, green and blue values
			     int newRed = (int) Math.round(red*factor);
			     int newGreen = (int) Math.round(green*factor);
			     int newBlue = (int) Math.round(blue*factor);

			     // put values for red, green and blue back into one RGB value
			     int newColor = newRed << 16 | newGreen << 8 | newBlue;

			     // overwrite data
			     data[offBuffer] = newColor;
			  }                                    
		}
		
		// increase count (for calculation of the next blend factor)
		if (count < frameCount)
	    	 count++;
		
		return BUFFER_PROCESSED_OK;
	}

	// not needed for this simple codec	
	public void open() throws ResourceUnavailableException {
	}

	// not needed for this simple codec
	public void close() {
	}

	// not needed for this simple codec
	public void reset() {
		close();
		try {
			open();
		} catch (ResourceUnavailableException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	// not needed for this simple codec
	public synchronized Object getControl(String controlType) {
		return null;
	}

	// not needed for this simple codec
	public synchronized Object[] getControls() {
		return new Object[0];
	}

	/**
	 * Return list of supported input format
	 * @return supported list of input format
	 */
	public Format[] getSupportedInputFormats() {
		return inputFormats;
	}
	
	/**
	 * Return list of supported output format for a given input format
	 * @param input input format
	 * @return supported list of output format for a given input format
	 */
	public Format[] getSupportedOutputFormats(Format input) {
		// if no input format specified return all output formats
		if (input == null)
			return (outputFormats);
		// only input RGBFormats are accepted
		if (input instanceof RGBFormat)
			// return new array, which includes the input and all output formats
			return (new Format[] { input, outputFormats[0] });
		else
			return (outputFormats);
	}

	/**
	 * Set input format (if RGBFormat) or returns the default input format
	 * @param input input format
	 * @return input format
	 */
	public Format setInputFormat(Format input) {
		// only input RGBFormats are accepted
		if (input instanceof RGBFormat)
			inputFormat = input;
		else
			// if specified format is not an RGBFormat return the default input format
			inputFormat = inputFormats[0];
		return (inputFormat);
	}

	/**
	 * Set output format (if RGBFormat) or returns the default output format
	 * @param output output format
	 * @return output format
	 */
	public Format setOutputFormat(Format output) {
		if (output instanceof RGBFormat)
			outputFormat = output;
		else
			outputFormat = outputFormats[0];
		return (outputFormat);
	}

	/**
	 * Return name that describes the effect
	 * @return name of effect
	 */
	public String getName() {
		return "blend effect";
	}
}
