import java.awt.Dimension;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.FileInputStream;

import javax.media.Buffer;
import javax.media.Effect;
import javax.media.Format;
import javax.media.ResourceUnavailableException;
import javax.media.format.RGBFormat;
import com.sun.image.codec.jpeg.JPEGCodec;
import com.sun.image.codec.jpeg.JPEGImageDecoder;

/**
 * @author Horst Eidenberger (hme@ims.tuwien.ac.at), adaptation: Heinrich
 *         Hussmann, Bettina Conradi, Sara Streng
 * @version 1
 */

public class LogoEffect implements Effect {

	/** supported input formats */
	private Format[] inputFormats;
	/** supported output formats */
	private Format[] outputFormats;
	
	//private LogoCodecControl control = new LogoCodecControl();
	private int [] logoArray = null;
	/** Logo image file name */
	private String logoFilename = "img/logo.jpg";
	/** Logo image file */
	private File logoFile = null;
	/** Width of the logo */
	private int logoWidth = 0;
	/** Height of the logo */
	private int logoHeight = 0;
	/** Position of logo on video (distance to the left) */
	private int offsetX = 10;
	/** Position of logo on video (distance to the top) */
	private int offsetY = 10;
	/** Transparency of the logo (1 = only logo, 0 = no logo visible) */
	private double alphaChannel = 0.5;

	public LogoEffect(File file) {
		this.logoFile = file;
		inputFormats = new Format[] { new RGBFormat(null, Format.NOT_SPECIFIED,
				Format.intArray, Format.NOT_SPECIFIED, 32,
				Format.NOT_SPECIFIED, Format.NOT_SPECIFIED,
				Format.NOT_SPECIFIED) };

		outputFormats = new Format[] { new RGBFormat(null,
				Format.NOT_SPECIFIED, Format.intArray, Format.NOT_SPECIFIED,
				32, Format.NOT_SPECIFIED, Format.NOT_SPECIFIED,
				Format.NOT_SPECIFIED) };
	}
	
	public LogoEffect() {
		inputFormats = new Format[] { new RGBFormat(null, Format.NOT_SPECIFIED,
				Format.intArray, Format.NOT_SPECIFIED, 32,
				Format.NOT_SPECIFIED, Format.NOT_SPECIFIED,
				Format.NOT_SPECIFIED) };

		outputFormats = new Format[] { new RGBFormat(null,
				Format.NOT_SPECIFIED, Format.intArray, Format.NOT_SPECIFIED,
				32, Format.NOT_SPECIFIED, Format.NOT_SPECIFIED,
				Format.NOT_SPECIFIED) };
	}

	/**
	 * This is where the actual effect is implemented. The method
	 * runs through all pixels (read out of the in buffer), 
	 * manipulates them and puts them back in the out buffer.
	 * @param in in buffer
	 * @param out out buffer
	 * @return status
	 */
	public synchronized int process(Buffer in, Buffer out) {
		
		// copy input buffer to output buffer
		out.copy(in);
		int[] data = (int[]) out.getData();
		
		// get information about the format
		RGBFormat inFormat = (RGBFormat) in.getFormat();
		Dimension inSize = inFormat.getSize();
		int width = (int) inSize.getWidth();
		//int height = (int) inSize.getHeight();	// not needed for the logo effect
		int redMask = inFormat.getRedMask();
		int greenMask = inFormat.getGreenMask();
		int blueMask = inFormat.getBlueMask();
		
		// pixel coordinates
		int x, y;
		// position of the pixel in the data[]
		int offBuffer;
		int offLogo, pixelData;

		// run through all pixels of the logo
		for(y=0; y<logoHeight; y++) {
			for(x=0; x<logoWidth; x++) {
				
				// add the offset (position of the logo)
				offBuffer = (y+offsetY)*width + offsetX + x;

				// position of the pixel in the data[]
				offLogo = (y * logoWidth + x);
			
				// calculate red = red(videoPixel)*(1-alpha) + red(logoPixel)*alpha
				// e.g. if alpha = 0.3
				// red = red(videoPixel)*0.7 + red(logoPixel)*0.3
				// & 0x000000ff = redMask = bit shifting
				pixelData =  ((int)((data[offBuffer] & redMask)   * (1-alphaChannel) + 
						(logoArray[offLogo] & 0x000000ff) * (alphaChannel))) & redMask;
				
				// add green = green(videoPixel)*(1-alpha) + green(logoPixel)*alpha
				// & 0x0000ff00 = greenMask = bit shifting
				pixelData += ((int)((data[offBuffer] & greenMask) * (1-alphaChannel) + 
						(logoArray[offLogo] & 0x0000ff00) * (alphaChannel))) & greenMask;
				
				// add blue = blue(videoPixel)*(1-alpha) + blue(logoPixel)*alpha
				// & 0x00ff0000 = blueMask = bit shifting
				pixelData += ((int)((data[offBuffer] & blueMask)  * (1-alphaChannel) + 
						(logoArray[offLogo] & 0x00ff0000) * (alphaChannel))) & blueMask;
				
				// overwrite data
				data[offBuffer] = pixelData;                                    
			}
		}

		// return the information that the processing was successful
		return BUFFER_PROCESSED_OK;
	}
			       
	public void open() throws ResourceUnavailableException {
		try {
			// load default image
			JPEGImageDecoder decoder;
			if(logoFile == null) {
				// load default image
				decoder = JPEGCodec.createJPEGDecoder(new FileInputStream(logoFilename));
			} else {
				// load image sent by user
				decoder = JPEGCodec.createJPEGDecoder(new FileInputStream(logoFile));
			}
			
			// convert to buffered image
			BufferedImage logoImg = decoder.decodeAsBufferedImage();
			
			// set logo width and height
			this.logoHeight = logoImg.getHeight();
			this.logoWidth = logoImg.getWidth();
			
			// read RGB
			logoArray = logoImg.getRGB(0, 0, logoWidth, logoHeight, null, 0, logoWidth);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	public void close() {
	}

	public void reset() {
		close();
		try {
			open();
		} catch (ResourceUnavailableException e) {
			e.printStackTrace();
		}
	}

	
	public synchronized Object getControl(String controlType) {
		return null;
	}

	public synchronized Object[] getControls() {
		return new Object[0];
	}

	/**
	 * Return list of supported input format
	 * @return supported list of input format
	 */
	public Format[] getSupportedInputFormats() {
		return inputFormats;
	}

	/**
	 * Return list of supported output format
	 * @param input input format
	 * @return supported list of output format for a given input format
	 */
	public Format[] getSupportedOutputFormats(Format input) {
		if (input == null)
			return (outputFormats);
		if (input instanceof RGBFormat)
			return (new Format[] { input, outputFormats[0] });
		else
			return (outputFormats);
	}

	/**
	 * Set input format (if RGBFormat) or returns the default input format
	 * @param input input format
	 * @return input format
	 */
	public Format setInputFormat(Format input) {
		// declare return value
		Format inputFormat;
		
		if (input instanceof RGBFormat)
			inputFormat = input;
		else
			inputFormat = inputFormats[0];
		return (inputFormat);
	}

	/**
	 * Set output format (if RGBFormat) or returns the default output format
	 * @param output output format
	 * @return output format
	 */
	public Format setOutputFormat(Format output) {
		// declare return value
		Format outputFormat;
		
		if (output instanceof RGBFormat)
			outputFormat = output;
		else
			outputFormat = outputFormats[0];
		return (outputFormat);
	}


	public String getName() {
		return "logo effect";
	}
}
