import java.awt.event.*;

import javax.swing.*;
import javax.swing.filechooser.FileFilter;
import javax.imageio.ImageIO;
import javax.imageio.ImageReader;
import javax.imageio.stream.ImageInputStream;
import javax.media.*;
import javax.media.format.*;
import javax.media.control.*;
import javax.media.protocol.*;
import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;

/**
 * @author Horst Eidenberger (hme@ims.tuwien.ac.at), adaptation: Heinrich
 *         Hussmann
 * @version 1
 */

/**
 * JFrame containing a video player that listens to ControllerEvents 
 * (processing chain related events).
 */
class VideoEffectView extends JFrame implements ControllerListener {

	/** serialVersionUID is needed because this class is serializable.*/
	private static final long serialVersionUID = 1L;
	/** Processor (any type of processing chain) */
	Processor processor = null;
	/** Player (processing chain restricted to rendering streams)*/
	Player player = null;
	/** Panel, which contains the video */
	JPanel videoPanel;
	/** Panel, which contains video effect controls */
	JPanel effectControlPanel;
	/** CheckBox for GrayEffect */
	JCheckBox grayEffectBox;
	/** CheckBox for BlendEffect */
	JCheckBox blendEffectBox;
	/** CheckBox for LogoEffect */
	JCheckBox logoEffectBox;
	/** Label specifying the name of the logo that is currently selected */
	JLabel logoFileLabel = new JLabel("");
	/** CheckBox for OtherEffect */
	JCheckBox additionalEffectBox;
	/** Slider for BlendEffect */
	JSlider framesPerSecond;
	/** Button that will open a file chooser to select a logo file */
	JButton loadLogoButton;
	/** File chooser for loading different logo images */
	final JFileChooser fc = new JFileChooser();
	/** Start Button */
	JButton startButton;
	/** File name */
	String filename = "";
	/** Logo file */
	File logoFile;
	/** True if processor is running, else false */
	boolean running = false;
	
	/**
	 * Creates a new VideoPlayerFrame and automatically loads the specified video file
	 * @param filename name of the video file
	 */
	public VideoEffectView(String filename) {
		
		// set up GUI
		setupView();
		
		// basic window functionality
		setTitle("Video Effects");
		addWindowListener(new WindowAdapter() {
			@Override
			public void windowClosing(WindowEvent event) {
				try {
					processor.stop();
					processor.deallocate();					
				}
				catch (Exception e) {
					System.out.println("Can't configure: " + e.getMessage());
				}
				System.exit(0);
			}
		});
		
		this.filename = filename;
	}
	
	private void setupView() {
		// add player to the content pane and do the layout
		getContentPane().setLayout(new BorderLayout());
		
		// create and add videoPanel
		videoPanel = new JPanel();
		videoPanel.setSize(450,400);
		videoPanel.setLayout(new BorderLayout());
		getContentPane().add(videoPanel, BorderLayout.WEST);

		// create and add effectControlPanel
		effectControlPanel = new JPanel(new GridLayout(6,1));
		effectControlPanel.setSize(100,400);
		JLabel label = new JLabel("Select effect(s):");
		effectControlPanel.add(label);
		grayEffectBox = new JCheckBox("Gray Effect");
		effectControlPanel.add(grayEffectBox);

		// ************** Blend effect controls **********************
		JPanel blendSettingPanel = new JPanel(new BorderLayout());
		//blendSettingPanel.setBorder(BorderFactory.createRaisedBevelBorder());
		blendEffectBox = new JCheckBox("Blend Effect");
		blendSettingPanel.add(blendEffectBox, BorderLayout.NORTH);
		
		// set slider values
		int FPS_MIN = 0;
		int FPS_MAX = 100;
		int FPS_INIT = 50;    //initial frames per second
		framesPerSecond = new JSlider(SwingConstants.HORIZONTAL,
		                                      FPS_MIN, FPS_MAX, FPS_INIT);

		// turn on labels at major tick marks
		framesPerSecond.setMajorTickSpacing(50);
		framesPerSecond.setMinorTickSpacing(10);
		framesPerSecond.setPaintTicks(true);
		framesPerSecond.setPaintLabels(true);

		blendSettingPanel.add(framesPerSecond, BorderLayout.SOUTH);
		effectControlPanel.add(blendSettingPanel);

		// ************** Logo effect controls **********************
		JPanel logoSettingPanel = new JPanel(new BorderLayout());
		//logoSettingPanel.setBorder(BorderFactory.createRaisedBevelBorder());
		logoEffectBox = new JCheckBox("Logo Effect");
		
		// set file filter -> only allow jpgs
		fc.setFileFilter(new FileFilter() {
            @Override
			public boolean accept(File f) {
            	boolean isJPEG = false;
            	
            	if (f.getName().toLowerCase().endsWith(".jpg") 
            			|| f.getName().toLowerCase().endsWith(".jpeg")
            			|| f.isDirectory()) {
            		isJPEG = true;
            	}            		
            	return isJPEG;
            }
            @Override
			public String getDescription() {
                return "Jpg Dateien(*.jpg)";
            }
        });

		
		// add a button that will open the file chooser when clicked
		loadLogoButton = new JButton("select logo");
		loadLogoButton.setSize(50, 20);
		loadLogoButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
			    //Handle open button action.
			    if (e.getSource() == loadLogoButton) {
			        int returnVal = fc.showOpenDialog(effectControlPanel);

			        if (returnVal == JFileChooser.APPROVE_OPTION) {
			        	// check if file is a jpeg
			            if(isJPEG(fc.getSelectedFile())) {
				            // set the file
				            logoFile = fc.getSelectedFile();
				            
				            // set label that shows the name of the 
				            // currently selected file
				            logoFileLabel.setText("selected file: " + logoFile.getName());
				            //logoFileLabel.setText("path/" + logoFile.getName());
				            
				            // automatically select logo effect
				            logoEffectBox.setSelected(true);
				            
			            } else {
				            logoFileLabel.setText("Error: Please choose a JPEG!");
			            }
			        } 
			   } 
			}
		});
		
		logoSettingPanel.add(logoEffectBox, BorderLayout.NORTH);
		logoSettingPanel.add(loadLogoButton, BorderLayout.CENTER);
		logoSettingPanel.add(logoFileLabel, BorderLayout.SOUTH);
		effectControlPanel.add(logoSettingPanel);
		
		// ************** Additional effect controls **********************
		additionalEffectBox = new JCheckBox("Your Effect");
		effectControlPanel.add(additionalEffectBox);
		
		
		// add a button that will start the processing when clicked
		startButton = new JButton("start");
		startButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				if(running == false) {
					startProcessing();
					startButton.setText("stop");
					running = true;
				}
				else {
					stopProcessing();
					startButton.setText("start");
					running = false;
				}
			}
		});
		effectControlPanel.add(startButton);
		
		getContentPane().add(effectControlPanel, BorderLayout.EAST);

		setSize(550, 400);
		setVisible(true);
	}
	
	private boolean isJPEG(File file) {
		boolean isJPEG = false;
    	
    	if (file.getName().toLowerCase().endsWith(".jpg") 
    			|| file.getName().toLowerCase().endsWith(".jpeg")) {
    		isJPEG = true;
    	}            		
    	return isJPEG;
	}

	
	public void startProcessing() {

		// create processor and control listener
		// catch exceptions in case something goes wrong
		try {
			// create a new processor and let the media locater located the input stream
			processor = Manager.createProcessor(new MediaLocator(filename));
			// tell the processor to send incoming events to this class (ControllerListener)
			processor.addControllerListener(this);
			
			processor.configure();
		} catch (Exception e) {
			System.out.println("Can't start the processor: " + e.getMessage());
		}
	}
	
	public void stopProcessing() {
		try {
			// stop player and processor
			player.stop();
			player.deallocate();
			processor.stop();
			processor.deallocate();		
			videoPanel.removeAll();
			videoPanel.repaint();
		} catch (Exception e) {
			System.out.println("Can't stop the processor: " + e.getMessage());
		}
	}

	/**
	 * Incoming events are sent to this method
	 * @param event incoming event
	 */
	public synchronized void controllerUpdate(ControllerEvent event) {

		// handle incoming ConfigureCompleteEvents
		if (event instanceof ConfigureCompleteEvent) {
			TrackControl[] tracks = processor.getTrackControls();

			// found is false until the first video track is found
			boolean found = false;
			
			// run through all tracks
			for (int i = 0; i < tracks.length; i++) {
				
				// get the first video track
				if (!found && tracks[i].getFormat() instanceof VideoFormat) {

					// define processing chain depending on checkboxes
					ArrayList<Codec> listArray = new ArrayList<Codec>();
					listArray.add(new com.sun.media.codec.video.cinepak.JavaDecoder());
					
					if(grayEffectBox.isSelected()) {
						listArray.add(new GrayEffect());
					}
					if (blendEffectBox.isSelected()) {
						listArray.add(new BlendEffect(framesPerSecond.getValue()));
					}
					if (logoEffectBox.isSelected()) {
						listArray.add(new LogoEffect(logoFile));
					}
					
					// convert ArrayList (with flexible size) to Codec[]
					Codec[] videoConversion = new Codec[listArray.size()];
					listArray.toArray(videoConversion);

					try {
						// apply processing chain to video track
						tracks[i].setCodecChain(videoConversion);
						
						// define format of output stream
						processor.setContentDescriptor(new ContentDescriptor(
								ContentDescriptor.RAW));
						
						// do not look for further video tracks
						found = true;
						
					} catch (Exception e) {
						System.out.println("Can't set processing chain: "
								+ e.getMessage());
					}
				} 
			}

			if (found)
				// go to next state -> realized
				processor.realize();
			else {
				System.out.println("Attempt to realize processor failed.");
				System.exit(-1);
			}

		}

		// handle incoming RealizeCompleteEvents
		else if (event instanceof RealizeCompleteEvent) {
			try {
				// get output from processor and feed it into a new realized player
				player = Manager.createRealizedPlayer(processor.getDataOutput());
				
				// add player to the content pane and do the layout
				this.videoPanel.add(player.getControlPanelComponent(),
						BorderLayout.SOUTH);
				this.videoPanel.add(player.getVisualComponent(),
						BorderLayout.CENTER);
				
				videoPanel.setSize(350, 300);
				setVisible(true);

				// start player and processor
				player.start();
				processor.start();
			} catch (Exception e) {
				System.out.println("Can't create player: " + e.getMessage());
			}
		}

		// handle incoming EndOfMediaEvents
		else if (event instanceof EndOfMediaEvent) {
			// stop player and processor
			player.stop();
			processor.stop();

			// restart player and processor
			processor.setMediaTime(new Time(0));
			player.setMediaTime(new Time(0));
			player.start();
			processor.start();
		}
	}

	public static void main(String[] argv) {
		String file = "http://www.medien.ifi.lmu.de/lehre/ss08/mt/uebungen/blatt4video.avi";
		new VideoEffectView(file);
	}
}
